﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Security.Cryptography;

namespace _64DDcheck
{
    class Program
    {
        static int[] BlockSizes = { 19720, 18360, 17680, 16320, 14960, 13600, 12240, 10880, 9520 };

        static int[] LBAs = {292, 292, 274, 274, 274, 274, 274, 204, 0,     //Head 0 (Zones 0-8) [+0]
                            0, 292, 292, 274, 274, 274, 274, 274, 204};     //Head 1 (Zones 0-8) [+9]

        static int[,] Zones =			//[Type][Zone]
            {
	            {0, 1, 2, 2, 1,
	            3, 4, 5, 6, 7, 8, 7, 6, 5, 4, 3},	//Type 0

            	{0, 1, 2, 3, 3, 2, 1,
	            4, 5, 6, 7, 8, 7, 6, 5, 4},		//Type 1

            	{0, 1, 2, 3, 4, 4, 3, 2, 1,
	            5, 6, 7, 8, 7, 6, 5},			//Type 2

            	{0, 1, 2, 3, 4, 5, 5, 4, 3, 2, 1,
	            6, 7, 8, 7, 6},				//Type 3

            	{0, 1, 2, 3, 4, 5, 6, 6, 5, 4, 3, 2, 1,
	            7, 8, 7},				//Type 4

            	{0, 1, 2, 3, 4, 5, 6, 7, 7, 6, 5, 4, 3, 2, 1,
            	8},					//Type 5

	            {0, 1, 2, 3, 4, 5, 6, 7, 8, 7, 6, 5, 4, 3, 2, 1}			//Type 6 (ROM only)
            };

        static int getROMLBAoffset(int LBAoffset, int type)
        {
            int offset = 0;
            int offset2 = 0;
            int offset3 = 0;

            for (int LBA = 0; LBA < LBAoffset; LBA++)
            {
                offset2 = 0;
                for (int i = 0; i <= offset3; i++)
                {
                    if (i <= (type + 2))
                        offset2 += LBAs[Zones[type, i]];
                    else
                        offset2 += LBAs[Zones[type, i] + 9];
                }

                if ((LBA >= offset2) && (offset3 < 5 + (type * 2)))
                    offset3++;

                offset += BlockSizes[Zones[type, offset3]];
            }
            return offset;
        }

        static void Main(string[] args)
        {
            Console.WriteLine("64DDcheck 0.1 by LuigiBlood\n----------------------------");

            if (args.Length == 0)
            {
                Console.WriteLine("USAGE: 64DDcheck <file>");
            }
            else
            {
                if (File.Exists(args[0]))
                {
                    //LBA 24 = Offset 0x738C0
                    byte[] system = new byte[232];
                    BinaryReader reader = new BinaryReader(File.Open(args[0], FileMode.Open));
                    reader.Read(system, 0, 232);

                    byte DiskType = system[0x05];
                    ushort LBAROMEnd = (ushort)((system[0xE0] << 8) ^ system[0xE1]);

                    Console.WriteLine("Disk Type: " + (DiskType & 0xF).ToString());
                    if ((DiskType & 0xF0) == 0x10)
                        Console.WriteLine("Retail Disk");
                    else
                        Console.WriteLine("Development Disk");

                    Console.WriteLine("LBA ROM End: " + (LBAROMEnd + 24).ToString());
                    int ROMsize = getROMLBAoffset((LBAROMEnd + 24 + 1), (DiskType & 0xF)) - 473280;

                    Console.WriteLine("ROM Area Size: " + ROMsize);

                    for (int i = 232; i < 473280; i++)
                        reader.ReadByte();

                    byte[] ROMbuffer = new byte[ROMsize];
                    reader.Read(ROMbuffer, 0, ROMsize);

                    SHA1CryptoServiceProvider cryptoTransformSHA1 = new SHA1CryptoServiceProvider();
                    string hash = BitConverter.ToString(cryptoTransformSHA1.ComputeHash(ROMbuffer)).Replace("-", "");

                    Console.WriteLine("ROM Area SHA1 Hash: " + hash);
                }
                else
                {
                    Console.WriteLine("This file does not exist!");
                    Environment.Exit(0);
                }
            }
            
        }
    }
}
